@@ -33,6 +33,7 @@ FALSE = jnp.bool_(False)
 # (2) we do not provide older versions (as with OpenAI Gym), and
 # (3) it is tedious to remember and write version numbers.
 EnvId = Literal[
+    "2048",
     # "animal_shogi",
     "backgammon",
     # "bridge_bidding",
@@ -205,6 +206,10 @@ def available_games() -> Tuple[EnvId, ...]:
 
 
 def make(env_id: EnvId):
+    if env_id == "2048":
+        from pgx.play2048 import Play2048
+
+        return Play2048()
     if env_id == "backgammon":
         from pgx.backgammon import Backgammon
 
