@@ -22,7 +22,6 @@ import pgx.core as core
 from pgx._cache import (
     load_shogi_is_on_the_way,
     load_shogi_legal_from_mask,
-    load_shogi_queen_moves,
     load_shogi_raw_effect_boards,
 )
 from pgx._flax.struct import dataclass
@@ -79,6 +78,7 @@ class State(core.State):
     reward: jnp.ndarray = jnp.float32([0.0, 0.0])
     terminated: jnp.ndarray = FALSE
     truncated: jnp.ndarray = FALSE
+    # action and observation are aligned to dlshogi https://github.com/TadaoYamaoka/DeepLearningShogi
     legal_action_mask: jnp.ndarray = jnp.zeros(27 * 81, dtype=jnp.bool_)
     observation: jnp.ndarray = jnp.zeros((119, 9, 9), dtype=jnp.bool_)
     _rng_key: jax.random.KeyArray = jax.random.PRNGKey(0)
@@ -87,11 +87,6 @@ class State(core.State):
     turn: jnp.ndarray = jnp.int8(0)  # 0 or 1
     piece_board: jnp.ndarray = INIT_PIECE_BOARD  # (81,) 後手のときにはflipする
     hand: jnp.ndarray = jnp.zeros((2, 7), dtype=jnp.int8)  # 後手のときにはflipする
-    # Not necessary. Cached only for Action.from_dlshogi_action
-    # Must be updated by `_legal_actions` if piece_board or hand is modified w/o `step`
-    legal_moves: jnp.ndarray = jnp.zeros((81, 81), dtype=jnp.bool_)
-    # Also cache
-    effects: jnp.ndarray = jnp.zeros((2, 81, 81), dtype=jnp.bool_)
 
     @staticmethod
     def _from_board(turn, piece_board: jnp.ndarray, hand: jnp.ndarray):
@@ -99,13 +94,11 @@ class State(core.State):
         terminated, reward, and current_player are not changed"""
         state = State(turn=turn, piece_board=piece_board, hand=hand)  # type: ignore
         # fmt: off
-        state = state.replace(effects=state.effects.at[0].set(_effects_all(state)))  # type: ignore
-        state = state.replace(effects=state.effects.at[1].set(_effects_all(_flip(state))))  # type: ignore
         state = jax.lax.cond(turn % 2 == 1, lambda: _flip(state), lambda: state)
         legal_moves, legal_promotions, legal_drops = _legal_actions(state)
         legal_action_mask = _to_direction(legal_moves, legal_promotions, legal_drops)
         # fmt: on
-        return state.replace(legal_action_mask=legal_action_mask, legal_moves=legal_moves)  # type: ignore
+        return state.replace(legal_action_mask=legal_action_mask)  # type: ignore
 
 
 class Shogi(core.Env):
@@ -156,8 +149,6 @@ IS_ON_THE_WAY = load_shogi_is_on_the_way()  # bool (5, 81, 81, 81)
 # x x x o x x x
 # x x o x x x x
 LEGAL_FROM_MASK = load_shogi_legal_from_mask()  # (10, 81, 81)
-# Queen piece does not exist in Shogi but necessary for computing legal moves
-QUEEN_MOVES = load_shogi_queen_moves()  # (81, 81)
 
 
 def _init_board():
@@ -185,16 +176,11 @@ def _init_board():
            [ 1,  2,  3,  6,  7,  6,  3,  2,  1]], dtype=int8)
     """
     state = State()
-    state = state.replace(effects=state.effects.at[0].set(_effects_all(state)))
-    state = state.replace(
-        effects=state.effects.at[1].set(_effects_all(_flip(state)))
-    )
     legal_moves, legal_promotions, legal_drops = _legal_actions(state)
     return state.replace(  # type: ignore
         legal_action_mask=_to_direction(
             legal_moves, legal_promotions, legal_drops
-        ),
-        legal_moves=legal_moves,
+        )
     )
 
 
@@ -265,6 +251,7 @@ class Action:
         action = jnp.int32(action)
         direction, to = jnp.int8(action // 81), jnp.int8(action % 81)
         is_drop = direction >= 20
+        is_promotion = (10 <= direction) & (direction < 20)
         # Compute <from> from <dir, to>
         #
         # LEGAL_FROM_MASK[UP, 18]  # to = 81
@@ -280,8 +267,9 @@ class Action:
         # x x o x o x x
         # x x x x x x x
         # x x x x x x x
-        mask1 = LEGAL_FROM_MASK[direction, to]  # (81,)
-        mask2 = state.legal_moves[:, to]  # (81,)
+        mask1 = LEGAL_FROM_MASK[direction % 10, to]  # (81,)
+        legal_moves, _, _ = _legal_actions(state)
+        mask2 = legal_moves[:, to]  # (81,)
         from_ = jnp.nonzero(mask1 & mask2, size=1)[0][0]
 
         piece = jax.lax.cond(
@@ -289,7 +277,6 @@ class Action:
             lambda: direction - 20,
             lambda: state.piece_board[from_],
         )
-        is_promotion = (10 <= direction) & (direction < 20)
         return Action(is_drop=is_drop, piece=piece, to=to, from_=from_, is_promotion=is_promotion)  # type: ignore
 
 
@@ -303,7 +290,7 @@ def _observe(state: State, player_id: jnp.ndarray) -> jnp.ndarray:
         my_pieces = jnp.arange(OPP_PAWN)
         my_piece_feat = jax.vmap(lambda p: state.piece_board == p)(my_pieces)
         # 自分の利き
-        my_effect = state.effects[0]
+        my_effect = _effects_all(state)
 
         def e(p):
             mask = state.piece_board == p
@@ -345,8 +332,9 @@ def _observe(state: State, player_id: jnp.ndarray) -> jnp.ndarray:
     my_hand_feat = hand_feat(state.hand[0])
     opp_hand_feat = hand_feat(state.hand[1])
 
+    flipped_effect_boards = _effects_all(_flip(state))
     checking_point_board, _ = _check_info(
-        state, _flip(state), state.effects[1]
+        state, _flip(state), flipped_effect_boards
     )
     checked = jnp.tile(checking_point_board.any(), reps=(1, 9, 9))
 
@@ -391,7 +379,6 @@ def _step(state: State, action: Action) -> State:
         reward=reward,
         terminated=terminated,
         legal_action_mask=legal_action_mask,
-        legal_moves=legal_moves,
     )
 
 
@@ -418,35 +405,7 @@ def _step_move(state: State, action: Action) -> State:
     # set piece to the target position
     pb = pb.at[action.to].set(piece)
     # apply piece moves
-    state = state.replace(piece_board=pb, hand=hand)  # type: ignore
-
-    ####################################################################################
-    # Update cached effects
-    ####################################################################################
-    # fmt: off
-    my_effects = state.effects[0]
-    opp_effects = state.effects[1]
-    # 移動元で塞がれていた利きを復元する（from_に効いていた大駒の利きを作り直す）
-    queen_effect = jnp.tile(_queen_effect(state, action.from_), reps=(81, 1))
-    my_effects |= _effect_filter_through(state, action.from_) & queen_effect
-    # 相手も同様
-    opp_effects |= (_effect_filter_through(_flip(state), _roatate_pos(action.from_)) & queen_effect[::-1])
-    # 移動元からの古い利きを消す
-    my_effects = my_effects.at[action.from_, :].set(FALSE)
-    # 移動先で相手の利きを消す（移動先で駒を取っていたら、取られた駒の利きを消す）
-    opp_effects = opp_effects.at[_roatate_pos(action.to), :].set(FALSE)
-    # 移動先から新しい利きを作る
-    my_effects = my_effects.at[action.to, :].set(_effects(state, action.to))
-    # 移動先を通るような利きを塞ぐ
-    my_effects &= ~_effect_filter_through(state, action.to)
-    # 相手も同様
-    opp_effects &= ~_effect_filter_through(_flip(state), _roatate_pos(action.to))
-    # set updated effects
-    state = state.replace(effects=state.effects.at[0].set(my_effects))  # type: ignore
-    state = state.replace(effects=state.effects.at[1].set(opp_effects))  # type: ignore
-    # fmt: on
-
-    return state
+    return state.replace(piece_board=pb, hand=hand)  # type: ignore
 
 
 def _step_drop(state: State, action: Action) -> State:
@@ -454,48 +413,38 @@ def _step_drop(state: State, action: Action) -> State:
     pb = state.piece_board.at[action.to].set(action.piece)
     # remove piece from hand
     hand = state.hand.at[0, action.piece].add(-1)
-    state = state.replace(piece_board=pb, hand=hand)  # type: ignore
-
-    ####################################################################################
-    # Update cached effects
-    ####################################################################################
-    # fmt: off
-    my_effects = state.effects[0]
-    opp_effects = state.effects[1]
-    # 新しい利きが増える
-    my_effects = my_effects.at[action.to, :].set(_effects(state, action.to))
-    # 打たれた点を経由する大駒の利きが消える
-    my_effects &= ~_effect_filter_through(state, action.to)
-    # 相手も同様
-    opp_effects &= ~_effect_filter_through(_flip(state), _roatate_pos(action.to))
-    # set updated effects
-    state = state.replace(effects=state.effects.at[0].set(my_effects))  # type: ignore
-    state = state.replace(effects=state.effects.at[1].set(opp_effects))  # type: ignore
-    # fmt: on
-
-    return state
+    return state.replace(piece_board=pb, hand=hand)  # type: ignore
 
 
 def _legal_actions(state: State):
-    effect_boards = state.effects[0]  # _apply_effects(state)
+    effect_boards = _effects_all(state)
     legal_moves = _pseudo_legal_moves(state, effect_boards)
     legal_drops = _pseudo_legal_drops(state, effect_boards)
 
     # Prepare necessary materials
     flipped_state = _flip(state)
-    flipped_effect_boards = state.effects[1]  # _apply_effects(flipped_state)
+    flipped_effect_boards = _effects_all(flipped_state)
     checking_point_board, check_defense_board = _check_info(
         state, flipped_state, flipped_effect_boards
     )
-    is_pinned = _find_pinned_pieces(state, flipped_state)
+    is_pinned, legal_pinned_moves = _find_pinned_pieces(state, flipped_state)
 
     # Filter illegal moves
-    legal_moves = _filter_suicide_moves(
-        state, legal_moves, flipped_effect_boards, is_pinned
+    legal_moves = _filter_suicide_king_moves(
+        state, legal_moves, flipped_effect_boards
     )
     legal_moves = _filter_ignoring_check_moves(
         state, legal_moves, checking_point_board, check_defense_board
     )
+    # filter pinned pieces' illegal moves
+    legal_moves_wo_pinned = jnp.where(
+        is_pinned.reshape(81, 1), FALSE, legal_moves
+    )
+    legal_moves = legal_moves_wo_pinned | (legal_moves & legal_pinned_moves)
+
+    legal_moves = _filter_double_check_moves(
+        state, legal_moves, checking_point_board
+    )
 
     # Filter illegal drops
     legal_drops = _filter_pawn_drop_mate(
@@ -523,8 +472,21 @@ def _pseudo_legal_moves(
     return effect_boards
 
 
-def _filter_suicide_moves(
-    state: State, legal_moves: jnp.ndarray, flipped_effect_boards, is_pinned
+def _filter_double_check_moves(state, legal_moves, checking_point_board):
+    # 両王手は王が動く以外ない
+    num_checks = checking_point_board.sum()
+    is_double_checked = num_checks > 1
+    king_mask = state.piece_board == KING
+    legal_moves = jax.lax.cond(
+        is_double_checked,
+        lambda: jnp.where(king_mask.reshape(81, 1), legal_moves, FALSE),  # type: ignore
+        lambda: legal_moves,
+    )
+    return legal_moves
+
+
+def _filter_suicide_king_moves(
+    state: State, legal_moves: jnp.ndarray, flipped_effect_boards
 ) -> jnp.ndarray:
     """Filter suicide action
      - King moves into the effected area
@@ -541,9 +503,6 @@ def _filter_suicide_moves(
     )
     legal_moves = jnp.where(mask, FALSE, legal_moves)
 
-    # pinned piece cannot move
-    legal_moves = jnp.where(is_pinned.reshape(81, 1), FALSE, legal_moves)
-
     return legal_moves
 
 
@@ -563,9 +522,19 @@ def _find_pinned_pieces(state, flipped_state):
             flipped_state.piece_board != EMPTY
         )
         return jax.lax.cond(
-            mask.sum() == 1, lambda: mask, lambda: jnp.zeros_like(mask)
+            (p >= 0) & (mask.sum() == 1),
+            lambda: mask,
+            lambda: jnp.zeros_like(mask),
         )
 
+    @jax.vmap
+    def on_the_way(p, f):
+        # fにあるpから王の間のマスクを返す
+        mask = (
+            IS_ON_THE_WAY[p, f, flipped_king_pos, :].at[f].set(TRUE)
+        )  # 王手をかけているコマも取れるのでTRUEをセット
+        return jax.lax.cond(p >= 0, lambda: mask, lambda: jnp.zeros_like(mask))
+
     from_ = jnp.arange(81)
     large_piece = _to_large_piece_ix(flipped_state.piece_board)
     # 利いてないところからの結果は無視する
@@ -576,7 +545,10 @@ def _find_pinned_pieces(state, flipped_state):
     ).any(axis=0)
     is_pinned = flipped_is_pinned[::-1]  # (81,)
 
-    return is_pinned
+    mask = on_the_way(large_piece, from_).any(axis=0)[::-1]
+    legal_pinned_piece_move = is_pinned.reshape(81, 1) & mask.reshape(1, 81)
+
+    return is_pinned, legal_pinned_piece_move
 
 
 def _filter_ignoring_check_moves(
@@ -803,7 +775,6 @@ def _flip(state: State):
     return state.replace(  # type: ignore
         piece_board=pb,
         hand=state.hand[jnp.int8((1, 0))],
-        effects=state.effects[jnp.int8((1, 0))],
     )
 
 
@@ -845,9 +816,9 @@ def _raw_effects_all(state: State) -> jnp.ndarray:
 
 
 def _to_large_piece_ix(piece):
-    # Filtering only Lance(0), Bishop(1), Rook(2), Horse(3), and Dragon(4)
+    # Filtering only Lance(0), Bishop(1), Rook(2), Horse(1), and Dragon(2)
     # NOTE: last 14th -1 is sentinel for avoid accessing via -1
-    return jnp.int8([-1, 0, -1, -1, 1, 2, -1, -1, -1, -1, -1, -1, 3, 4, -1])[
+    return jnp.int8([-1, 0, -1, -1, 1, 2, -1, -1, -1, -1, -1, -1, 1, 2, -1])[
         piece
     ]
 
@@ -890,134 +861,6 @@ def _effect_filters_all(state: State) -> jnp.ndarray:
     return filter_boards  # (81=from, 81=to)
 
 
-def _effect_filter_through(
-    state: State, blocked_pos: jnp.ndarray
-) -> jnp.ndarray:
-    """
-    >>> s = _init_board()
-    >>> _rotate(_effect_filter_through(s, 10).any(axis=0))
-    Array([[False, False, False, False, False, False, False,  True,  True],
-           [False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False, False]],      dtype=bool)
-    """
-    pieces = state.piece_board
-    large_pieces = jax.vmap(_to_large_piece_ix)(pieces)
-    from_ = jnp.arange(81)
-    to = jnp.arange(81)
-
-    def func(p, f):
-        # pieceがfromからtoへblock_posに妨害されずに到達できるか否か
-        return IS_ON_THE_WAY[p, f, to, blocked_pos]
-
-    # (piece, from) にバッチで適用
-    filter_boards = jax.vmap(func)(large_pieces, from_)  # (81, 81)
-
-    mask = (large_pieces >= 0).reshape(81, 1)
-    filter_boards = jnp.where(mask, filter_boards, FALSE)
-
-    return filter_boards  # (81=from, 81=to)
-
-
-def _effect_filter_from(state: State, from_: jnp.ndarray) -> jnp.ndarray:
-    """
-    >>> s = _init_board()
-    >>> _rotate(_effect_filter_from(s, 8))
-    Array([[False, False, False, False, False, False, False, False,  True],
-           [False, False, False, False, False, False, False, False,  True],
-           [False, False, False, False, False, False, False, False,  True],
-           [False, False, False, False, False, False, False, False,  True],
-           [False, False, False, False, False, False, False, False,  True],
-           [False, False, False, False, False, False, False, False,  True],
-           [False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False, False]],      dtype=bool)
-    """
-    piece = state.piece_board[from_]
-    large_piece = _to_large_piece_ix(piece)
-    to = jnp.arange(81)
-
-    def func(t):
-        # toを固定したとき、pieceがfromからtoへ妨害されずに到達できるか否か
-        # True = 途中でbrockされ、到達できない
-        return (
-            IS_ON_THE_WAY[large_piece, from_, t, :] & (state.piece_board >= 0)
-        ).any()
-
-    # (piece, from) にバッチで適用
-    filter_boards = jax.vmap(func)(to)  # (81,81)
-
-    # from_にある駒が大駒でないとき、利きには何も影響がない
-    filter_boards = jax.lax.cond(
-        large_piece == -1,
-        lambda: jnp.zeros((81,), dtype=jnp.bool_),
-        lambda: filter_boards,
-    )
-
-    return filter_boards  # (81=to)
-
-
-def xy2i(x, y):
-    """
-    >>> xy2i(2, 6)  # 26歩
-    14
-    """
-    i = (x - 1) * 9 + (y - 1)
-    return i
-
-
-def _queen_effect(state: State, from_: jnp.ndarray) -> jnp.ndarray:
-    """
-    >>> s = _init_board()
-    >>> _rotate(_queen_effect(s, xy2i(5, 5)))
-    Array([[False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False, False],
-           [False, False,  True, False,  True, False,  True, False, False],
-           [False, False, False,  True,  True,  True, False, False, False],
-           [ True,  True,  True,  True, False,  True,  True,  True,  True],
-           [False, False, False,  True,  True,  True, False, False, False],
-           [False, False,  True, False,  True, False,  True, False, False],
-           [False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False, False]],      dtype=bool)
-    """
-    queen_moves = QUEEN_MOVES[from_, :]
-
-    def filter(t):
-        # queenはrookとbishopのor
-        is_occupied = state.piece_board >= 0
-        bishop_filter = (IS_ON_THE_WAY[1, from_, t, :] & is_occupied).any()
-        rook_filter = (IS_ON_THE_WAY[2, from_, t, :] & is_occupied).any()
-        return bishop_filter | rook_filter
-
-    filter_board = jax.vmap(filter)(jnp.arange(81))  # (81,)
-    return queen_moves & ~filter_board  # (81,)
-
-
-def _effects(state: State, from_: jnp.ndarray):
-    """fromに現在ある駒からの利き
-
-    >>> s = _init_board()
-    >>> _rotate(_effects(s, 8))  # 香
-    Array([[False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False, False],
-           [False, False, False, False, False, False, False, False,  True],
-           [False, False, False, False, False, False, False, False,  True],
-           [False, False, False, False, False, False, False, False, False]],      dtype=bool)
-    """
-    raw_effect_boards = RAW_EFFECT_BOARDS[state.piece_board[from_], from_, :]
-    effect_filter_boards = _effect_filter_from(state, from_)
-    return raw_effect_boards & ~effect_filter_boards
-
-
 def _effects_all(state: State):
     """
     >>> s = _init_board()
