@@ -4,12 +4,22 @@ import jax.numpy as jnp
 from pgx.shogi import *
 from pgx.shogi import _step, _step_move, _step_drop, _flip, _effects_all, _legal_actions, _rotate, _to_direction, _from_sfen, _pseudo_legal_drops, _to_sfen
 
+
 env = Shogi()
 init = jax.jit(env.init)
 step = jax.jit(env.step)
 observe = jax.jit(env.observe)
 
 
+def xy2i(x, y):
+    """
+    >>> xy2i(2, 6)  # 26歩
+    14
+    """
+    i = (x - 1) * 9 + (y - 1)
+    return i
+
+
 # check visualization results by image preview plugins
 def visualize(state, fname="tests/assets/shogi/xxx.svg"):
     from pgx._visualizer import Visualizer
@@ -21,9 +31,6 @@ def update_board(state, piece_board, hand=None):
     state = state.replace(piece_board=piece_board)
     if hand is not None:
         state = state.replace(hand=hand)
-    state = state.replace(effects=state.effects.at[0].set(_effects_all(state)))
-    state = state.replace(effects=state.effects.at[1].set(_effects_all(_flip(state))))
-    state = state.replace(legal_moves=_legal_actions(state)[0])
     return state
 
 
@@ -163,6 +170,7 @@ def test_legal_moves():
     visualize(s, "tests/assets/shogi/legal_moves_004.svg")
     legal_moves, _, _ = _legal_actions(s)
     assert not legal_moves[xy2i(5, 7), xy2i(4, 6)]
+    assert legal_moves[xy2i(5, 7), xy2i(5, 6)]
 
     # Gold is not pinned
     key = jax.random.PRNGKey(0)
@@ -540,6 +548,61 @@ def test_buggy_samples():
     visualize(s, "tests/assets/shogi/buggy_samples_004.svg")
     assert (jnp.nonzero(s.legal_action_mask)[0] == jnp.int32([43, 52, 68, 196, 222, 295, 789, 1996, 2004, 2012])).all()
 
+    # #602
+    sfen = "9/4R4/9/9/9/9/9/9/9 b 2r2b4g3s4n4l17p 1"
+    state = _from_sfen(sfen)
+    visualize(state, "tests/assets/shogi/buggy_samples_005.svg")
+    dlshogi_action = 846
+    state = step(state, dlshogi_action)
+    visualize(state, "tests/assets/shogi/buggy_samples_006.svg")
+    sfen = "4+R4/9/9/9/9/9/9/9/9 w 2r2b4g3s4n4l7p 1"
+    expected_state = _from_sfen(sfen)
+    visualize(expected_state, "tests/assets/shogi/buggy_samples_006.svg")
+    assert (state.piece_board == expected_state.piece_board).all()
+
+    # #603
+    state = _from_sfen("8k/9/9/5b3/9/3B5/9/9/K8 b 2r4g4s4n4l18p 1")
+    visualize(state, "tests/assets/shogi/buggy_samples_007.svg")
+    dlshogi_action = 202
+    a = Action.from_dlshogi_action(state, dlshogi_action)
+    assert a.from_ == xy2i(6, 6)
+
+    # #610
+    state = _from_sfen("+PsGg1p2+P/+B1+Pgp+N1sp/1+N5l1/P3kP1pL/3P1r3/B2KP3L/4L1SP+s/+r2+p2pgP/2P2+n+p2 b np 1")
+    visualize(state, "tests/assets/shogi/buggy_samples_008.svg")
+    dlshogi_action = 225
+    a = Action.from_dlshogi_action(state, dlshogi_action)
+    effects = _effects_all(state)
+    assert effects[xy2i(9, 2), xy2i(8, 1)]
+    assert a.from_ == xy2i(9, 2)
+    assert a.piece == HORSE
+    state = step(state, dlshogi_action)
+    expected_state = _from_sfen("+P+BGg1p2+P/2+Pgp+N1sp/1+N5l1/P3kP1pL/3P1r3/B2KP3L/4L1SP+s/+r2+p2pgP/2P2+n+p2 w Snp 1")
+    assert (state.piece_board == expected_state.piece_board).all()
+
+    # #613
+    state = _from_sfen("1+N3s1n1/5k2l/l+P2g1bp1/2pP1p2p/p2ppNS2/LB6P/1pS1g2PL/3KPR2S/1R1G1NG2 b P4p 1")
+    visualize(state, "tests/assets/shogi/buggy_samples_009.svg")
+    dlshogi_action = 42
+    a = Action.from_dlshogi_action(state, dlshogi_action)
+    assert a.from_ == xy2i(5, 8)
+    assert a.piece == PAWN
+    state = step(state, dlshogi_action)
+    expected_state = _from_sfen("1+N3s1n1/5k2l/l+P2g1bp1/2pP1p2p/p2ppNS2/LB6P/1pS1P2PL/3K1R2S/1R1G1NG2 w GP4p 1")
+    assert (state.piece_board == expected_state.piece_board).all()
+
+    # #618
+    state = _from_sfen("2+P+P2G1+S/1P2+P+P1+Pn/+S1GK2P2/1b2PP3/1nl4PP/3k2lRL/1pg+s3L1/p2R2p2/P+n+B+p+ng1+s+p w P 1")
+    visualize(state, "tests/assets/shogi/buggy_samples_010.svg")
+    dlshogi_action = 28
+    legal_moves, *_ = _legal_actions(state)
+    assert legal_moves[xy2i(4, 3), xy2i(4, 2)]
+    a = Action.from_dlshogi_action(state, dlshogi_action)
+    assert a.from_ == xy2i(4, 3)
+    state = step(state, dlshogi_action)
+    expected_state = _from_sfen("2+P+P2G1+S/1P2+P+P1+Pn/+S1GK2P2/1b2PP3/1nl4PP/3k2lRL/1pg4L1/p2+s2p2/P+n+B+p+ng1+s+p b Pr 1")
+    assert (state.piece_board == expected_state.piece_board).all()
+
 
 def test_observe():
     key = jax.random.PRNGKey(0)
